/***************************************************************************
                          kcoverpage.cpp  -  description
                             -------------------
    begin                : Sat Apr 15 2000
    copyright            : (C) 2000 by Pascal 'PeP' Panneels
    email                : pp@bvdep.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kcoverpage.h"

KCoverPage::KCoverPage()
{
	NumberOfObjects=0;
	PageToPrint = false;
}

KCoverPage::~KCoverPage()
{
}

KCoverObject * KCoverPage::GetSelectedObject(QPoint &p)
{
	// look in the multi strings list, scanning from top to beyond
	for (std::list<KCoverMultiString>::reverse_iterator i=MultiStrings.rbegin(); i!=MultiStrings.rend();i++)
		if ( (*i).IsPointInObject(p) )
			return &(*i);
	
	// look in the curved strings list, scanning from top to beyond
	for (std::list<KCoverCurvedString>::reverse_iterator i=CurvedStrings.rbegin(); i!=CurvedStrings.rend();i++)
		if ( (*i).IsPointInObject(p) )
			return &(*i);
	
	// look in the strings list, scanning from top to beyond
	for (std::list<KCoverString>::reverse_iterator i=Strings.rbegin(); i!=Strings.rend();i++)
		if ( (*i).IsPointInObject(p) )
			return &(*i);
	
	// look in the images list, scanning from top to beyond
	for (std::list<KCoverImage>::reverse_iterator i=Images.rbegin(); i!=Images.rend();i++)
		//if ( (p.x()>=(*i).X) && (p.x()<=(*i).X+(*i).Image.width()) && (p.y()>=(*i).Y) && (p.y()<=(*i).Y+(*i).Image.height()) )
		if ( (*i).IsPointInObject(p) )
			return &(*i);
		
	return NULL;
}

/** add an image to the list */
void KCoverPage::AddImage(KCoverImage img)
{
	bool cont = true;
	std::list<KCoverImage>::iterator j=Images.begin();
	
    while ( (img.Order > (*j).Order) && cont && ( j!=Images.end()) )
    	j++;
	Images.insert(j, img);
	NumberOfObjects++;
}

/** return the image #i of the list */
KCoverImage * KCoverPage::GetImage(int i)
{
	std::list<KCoverImage>::iterator j=Images.begin();
	for (; i>0; i--)
		j++;
	return &(*j);
}

/** Erase the selected object */
void KCoverPage::DeleteSelectedObject(KCoverObject *obj)
{
	switch ( obj->GetClassType() )
		{
		case TYPE_CLASS_IMAGE :
			{
			KCoverImage * img = dynamic_cast<KCoverImage *>(obj);
			std::list<KCoverImage>::iterator j;
			j = std::find(Images.begin(), Images.end(), *img );
			Images.erase(j);
			NumberOfObjects--;
			} break;
		case TYPE_CLASS_STRING :
			{
			KCoverString * str = dynamic_cast<KCoverString *>(obj);
			std::list<KCoverString>::iterator j;
			j = std::find(Strings.begin(), Strings.end(), *str );
			Strings.erase(j);
			NumberOfObjects--;
			} break;
		case TYPE_CLASS_CURVED_STRING :
			{
			KCoverCurvedString * str = dynamic_cast<KCoverCurvedString *>(obj);
			std::list<KCoverCurvedString>::iterator j;
			j = std::find(CurvedStrings.begin(), CurvedStrings.end(), *str );
			CurvedStrings.erase(j);
			NumberOfObjects--;
			} break;
		case TYPE_CLASS_MULTI_STRING :
			{
			KCoverMultiString * str = dynamic_cast<KCoverMultiString *>(obj);
			std::list<KCoverMultiString>::iterator j;
			j = std::find(MultiStrings.begin(), MultiStrings.end(), *str );
			MultiStrings.erase(j);
			NumberOfObjects--;
			} break;
		}
}

/**  sort the image list */
void KCoverPage::SortImages()
{
	Images.sort();
}

/** add a string to the page */
void KCoverPage::AddString(KCoverString str)
{
	bool cont = true;
	std::list<KCoverString>::iterator j=Strings.begin();
	
    while ( (str.Order > (*j).Order) && cont && ( j!=Strings.end()) )
    	j++;
	Strings.insert(j, str);
	NumberOfObjects++;
}

/** sort the string list */
void KCoverPage::SortStrings()
{
	Strings.sort();
}

/** return the string number i */
KCoverString * KCoverPage::GetString(int i)
{
	std::list<KCoverString>::iterator j=Strings.begin();
	for (; i>0; i--)
		j++;
	return (&(*j));
}

/** clear the page content */
void KCoverPage::Clear()
{
	Images.erase( Images.begin(), Images.end() );
	Strings.erase( Strings.begin(), Strings.end() );
	CurvedStrings.erase( CurvedStrings.begin(), CurvedStrings.end() );
	MultiStrings.erase( MultiStrings.begin(), MultiStrings.end() );

	NumberOfObjects = 0;
	PageToPrint = false;
}

/** add a curved string to the page */
void KCoverPage::AddCurvedString(KCoverCurvedString str)
{
	bool cont = true;
	std::list<KCoverCurvedString>::iterator j=CurvedStrings.begin();
	
    while ( (str.Order > (*j).Order) && cont && ( j!=CurvedStrings.end()) )
    	j++;
	CurvedStrings.insert(j, str);
	NumberOfObjects++;
}

/** sort the curved string list */
void KCoverPage::SortCurvedStrings()
{
	CurvedStrings.sort();
}

/** return the string number i */
KCoverCurvedString * KCoverPage::GetCurvedString(int i)
{
	std::list<KCoverCurvedString>::iterator j=CurvedStrings.begin();
	for (; i>0; i--)
		j++;
	return (&(*j));
}

/** add a multi string to the page */
void KCoverPage::AddMultiString(KCoverMultiString str)
{
	bool cont = true;
	std::list<KCoverMultiString>::iterator j=MultiStrings.begin();
	
    while ( (str.Order > (*j).Order) && cont && ( j!=MultiStrings.end()) )
    	j++;
	MultiStrings.insert(j, str);
	NumberOfObjects++;
}

/** sort the multi string list */
void KCoverPage::SortMultiStrings()
{
	MultiStrings.sort();
}

/** return the string number i */
KCoverMultiString * KCoverPage::GetMultiString(int i)
{
	std::list<KCoverMultiString>::iterator j=MultiStrings.begin();
	for (; i>0; i--)
		j++;
	return (&(*j));
}

/** save a page */
void KCoverPage::Save(QFile &file, int pagenumber)
{
	char tmp[20];
	unsigned int nb;
	
	if ( !NumberOfObjects )
		return;
		
	tmp[0] = TAG_PAGE;
	file.writeBlock( tmp, 1 );
	file.writeBlock( (char *)&pagenumber, sizeof(int) );
	// save the strings
	nb = Strings.size();
	file.writeBlock( (char *)&nb, sizeof(int) );
	if ( nb>0 )
		for ( std::list<KCoverString>::iterator i=Strings.begin(); i!=Strings.end(); i++ )
			i->Save(file);
	// save the mutli line strings
	nb = MultiStrings.size();
	file.writeBlock( (char *)&nb, sizeof(int) );
	if ( nb>0 )
		for ( std::list<KCoverMultiString>::iterator i=MultiStrings.begin(); i!=MultiStrings.end(); i++ )
			i->Save(file);
	// save the curved strings
	nb = CurvedStrings.size();
	file.writeBlock( (char *)&nb, sizeof(int) );
	if ( nb>0 )
		for ( std::list<KCoverCurvedString>::iterator i=CurvedStrings.begin(); i!=CurvedStrings.end(); i++ )
			i->Save(file);
	// save the images
	nb = Images.size();
	file.writeBlock( (char *)&nb, sizeof(int) );
	if ( nb>0 )
		for ( std::list<KCoverImage>::iterator i=Images.begin(); i!=Images.end(); i++ )
			i->Save(file);
			
}

/** load a page */
void KCoverPage::Load(QFile &file, int version)
{
	unsigned int nb;
	int err;
	
	err = file.readBlock((char *)&nb, sizeof(int));
	for (unsigned int i=0; i<nb; i++ )
		{
		KCoverString *str = new KCoverString;
		str->Load(file, version);
		AddString( *str );
	    }
	
	err = file.readBlock((char *)&nb, sizeof(int));
	for (unsigned int i=0; i<nb; i++ )
		{
		KCoverMultiString *str = new KCoverMultiString;
		str->Load(file, version);
		AddMultiString( *str );
	    }

	err = file.readBlock((char *)&nb, sizeof(int));
	for (unsigned int i=0; i<nb; i++ )
		{
		KCoverCurvedString *str = new KCoverCurvedString;
		str->Load(file, version);
		AddCurvedString( *str );
	    }
	err = file.readBlock((char *)&nb, sizeof(int));
	for (unsigned int i=0; i<nb; i++ )
		{
		KCoverImage *str = new KCoverImage;
		str->Load(file, version);
		AddImage( *str );
	    }
	    	
}

