/***************************************************************************
                          kcoverstring.cpp  -  description
                             -------------------
    begin                : Mon Apr 3 2000
    copyright            : (C) 2000 by Pascal 'PeP' Panneels
    email                : pp@bvdep.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kcoverstring.h"
#include <qfontmetrics.h>

KCoverString::KCoverString(string text, int x, int y, int order ) : KCoverObject( x, y, order )
{
	Text = text;
	Font = new KCoverFont("Helvetica");
}

KCoverString::~KCoverString()
{
}

int KCoverString::operator == (const KCoverString &p) const
{
	if (p.X==X && p.Y==Y && p.Order==Order && p.Text==Text )
		return 1;
	else
		return 0;
}

/** return true if clicked is on a text */
bool KCoverString::IsPointInObject(QPoint &pos)
{
	QFontMetrics fm(*Font->Font);
	int W = fm.width(Text.c_str());
	int H = fm.height();
	
	if ( pos.x()>=X && pos.x()<=X+W && pos.y()<=Y && pos.y()>=Y-H )
		return true;
	else
		return false;
}

int KCoverString::Width()
{
	QFontMetrics fm(*Font->Font);
	return fm.width(Text.c_str());
}

int KCoverString::Height()
{
	QFontMetrics fm(*Font->Font);
	fm.width(Text.c_str());
	return fm.height();
}

int KCoverString::RealX()
{
	QFontMetrics fm(*Font->Font);
	QRect r = fm.boundingRect(Text.c_str());
	return X+r.x();
}

int KCoverString::RealY()
{
	QFontMetrics fm(*Font->Font);
	QRect r = fm.boundingRect(Text.c_str());
	return Y+r.y();
}

void KCoverString::Paint( QPainter &paint,  QPoint orig)
{
	QPoint pos=QPoint(X,Y)+orig;
    paint.setFont(*Font->Font);
    paint.setPen(*textcolor);
    paint.setBackgroundMode(transparentbackground ? Qt::TransparentMode : Qt::OpaqueMode );
    paint.setBackgroundColor(*backcolor);    		
    paint.drawText( pos, Text.c_str() );
}

/** Save a string */
void KCoverString::Save(QFile &file)
{
	char tmp[256];
	int r1, g1, b1, r2, g2, b2;
	
	// tag TAG_STRING
	tmp[0] = TAG_STRING;
	file.writeBlock( tmp, 1 );
	// call the base class to save the coordinates
	KCoverObject::Save(file);
	// save colors, alignement, transparent info
	backcolor->rgb( &r1, &g1, &b1 );
	textcolor->rgb( &r2, &g2, &b2 );
	sprintf( tmp,"%c%c%c%c%c%c%c%c", alignoption, transparentbackground, r1, g1, b1, r2, g2, b2 );
	file.writeBlock( tmp,8 );
	// save the font information
	Font->Save( file );	
	// save the text itself
	r1 = Text.size();
	file.writeBlock( (char *)&r1, sizeof(int) );
	file.writeBlock( Text.c_str(), r1 );
}

/** load an object */
void KCoverString::Load(QFile &file, int version)
{
	char tmp[256];
	int r1;
	int err;
	
	err = file.readBlock( tmp, 1 );
	if ( tmp[0]!= TAG_STRING )
		return; // error : not a string
	KCoverObject::Load(file, version);
	err = file.readBlock( tmp, 8 );
	alignoption = tmp[0];
	transparentbackground = tmp[1];
	backcolor = new QColor( (unsigned char)tmp[2], (unsigned char)tmp[3], (unsigned char)tmp[4] );
	textcolor = new QColor( (unsigned char)tmp[5], (unsigned char)tmp[6], (unsigned char)tmp[7] );
	
	Font = new KCoverFont;
	Font->Load(file,version);	
	err = file.readBlock( (char *)&r1, sizeof(int) );
	err = file.readBlock( tmp, r1 );
	tmp[r1] = 0;
	Text = string( tmp );
}
